Package: cmstatr
Type: Package
Title: Statistical Methods for Composite Material Data
Version: 0.10.0
Date: 2024-11-18
Depends: R (>= 3.3)
Authors@R: c(
    person("Stefan", "Kloppenborg", email = "stefan@kloppenborg.ca",
      comment = c(ORCID = "0000-0002-1908-5214"),
      role = c("aut", "cre")),
    person("Billy", "Cheng", email = "bcheng@comtekadvanced.com",
      role = c("ctb")),
    person("Ally", "Fraser", email = "ally.fraser25@gmail.com",
      role = c("ctb")),
    person("Jeffrey", "Borlik",
      role = c("ctb")),
    person("Brice", "Langston", email = "tblangst@gmail.com",
      role = c("ctb")),
    person("Comtek Advanced Structures, Ltd.",
      role = c("fnd"))
  )
Description: An implementation of the statistical methods commonly
  used for advanced composite materials in aerospace applications.
  This package focuses on calculating basis values (lower tolerance
  bounds) for material strength properties, as well as performing the
  associated diagnostic tests. This package provides functions for
  calculating basis values assuming several different distributions,
  as well as providing functions for non-parametric methods of computing
  basis values. Functions are also provided for testing the hypothesis
  that there is no difference between strength and modulus data from an
  alternate sample and that from a "qualification" or "baseline" sample.
  For a discussion of these statistical methods and their use, see the
  Composite Materials Handbook, Volume 1 (2012, ISBN: 978-0-7680-7811-4).
  Additional details about this package are available in the paper by
  Kloppenborg (2020, <doi:10.21105/joss.02265>).
URL: https://www.cmstatr.net/, https://github.com/cmstatr/cmstatr
BugReports: https://github.com/cmstatr/cmstatr/issues
License: AGPL-3
Encoding: UTF-8
LazyData: true
Imports: dplyr, generics, ggplot2, kSamples, MASS, purrr, rlang, stats,
        tibble, tidyr
Suggests: knitr, lintr, rmarkdown, spelling, testthat, vdiffr
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Language: en-US
Config/testthat/parallel: true
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2024-11-19 04:04:50 UTC; stefan
Author: Stefan Kloppenborg [aut, cre] (<https://orcid.org/0000-0002-1908-5214>),
  Billy Cheng [ctb],
  Ally Fraser [ctb],
  Jeffrey Borlik [ctb],
  Brice Langston [ctb],
  Comtek Advanced Structures, Ltd. [fnd]
Maintainer: Stefan Kloppenborg <stefan@kloppenborg.ca>
Repository: CRAN
Date/Publication: 2024-11-19 07:50:08 UTC
Built: R 4.6.0; ; 2026-04-29 14:06:25 UTC; unix
