# Generated by roxygen2: do not edit by hand

S3method(augment,mnr)
S3method(condition_summary,basis)
S3method(condition_summary,data.frame)
S3method(glance,adk)
S3method(glance,anderson_darling)
S3method(glance,basis)
S3method(glance,equiv_change_mean)
S3method(glance,equiv_mean_extremum)
S3method(glance,levene)
S3method(glance,mnr)
S3method(print,adk)
S3method(print,anderson_darling)
S3method(print,basis)
S3method(print,equiv_change_mean)
S3method(print,equiv_mean_extremum)
S3method(print,levene)
S3method(print,mnr)
export(ad_ksample)
export(anderson_darling_lognormal)
export(anderson_darling_normal)
export(anderson_darling_weibull)
export(augment)
export(basis_anova)
export(basis_hk_ext)
export(basis_lognormal)
export(basis_nonpara_large_sample)
export(basis_normal)
export(basis_pooled_cv)
export(basis_pooled_sd)
export(basis_weibull)
export(calc_cv_star)
export(condition_summary)
export(cv)
export(equiv_change_mean)
export(equiv_mean_extremum)
export(geom_jitter_failure_mode)
export(glance)
export(hk_ext_z)
export(hk_ext_z_j_opt)
export(k_equiv)
export(k_factor_normal)
export(levene_test)
export(maximum_normed_residual)
export(nested_data_plot)
export(nonpara_binomial_rank)
export(normalize_group_mean)
export(normalize_ply_thickness)
export(separate_failure_modes)
export(stat_esf)
export(stat_normal_surv_func)
export(tidy)
export(transform_mod_cv)
export(transform_mod_cv_ad)
importFrom(MASS,fitdistr)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,ungroup)
importFrom(generics,augment)
importFrom(generics,glance)
importFrom(generics,tidy)
importFrom(ggplot2,GeomPoint)
importFrom(ggplot2,Stat)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,expansion)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggproto)
importFrom(ggplot2,labs)
importFrom(ggplot2,layer)
importFrom(ggplot2,position_jitter)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(kSamples,ad.test)
importFrom(purrr,map_dfr)
importFrom(rlang,.data)
importFrom(rlang,abort)
importFrom(rlang,call2)
importFrom(rlang,call_args)
importFrom(rlang,call_args_names)
importFrom(rlang,call_name)
importFrom(rlang,enquo)
importFrom(rlang,enquos)
importFrom(rlang,ensym)
importFrom(rlang,eval_tidy)
importFrom(rlang,exec)
importFrom(rlang,fn_fmls_names)
importFrom(rlang,inform)
importFrom(rlang,list2)
importFrom(rlang,quo_get_expr)
importFrom(rlang,quo_text)
importFrom(rlang,warn)
importFrom(stats,dbeta)
importFrom(stats,dnorm)
importFrom(stats,integrate)
importFrom(stats,median)
importFrom(stats,pbeta)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,pnorm)
importFrom(stats,pweibull)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,qweibull)
importFrom(stats,sd)
importFrom(stats,uniroot)
importFrom(stats,var.test)
importFrom(tibble,tibble)
importFrom(tidyr,separate_rows)
