
#' Sample data for a generic carbon fabric
#'
#' Datasets containing sample data that is typical of a generic carbon
#' fabric prepreg. This data is used in several examples within the
#' `cmstatr` package. This data is fictional and should
#' only be used for learning how to use this package.
#'
#' @name carbon.fabric

#' @rdname carbon.fabric
"carbon.fabric"

#' @rdname carbon.fabric
"carbon.fabric.2"
