#' @importFrom generics augment
#' @export
#' @seealso [generics::augment()]
generics::augment

#' @importFrom generics tidy
#' @export
#' @seealso [generics::tidy()]
generics::tidy

#' @importFrom generics glance
#' @export
#' @seealso [generics::glance()]
generics::glance
